/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package data;

import java.io.Serializable;

/**
 *
 * @author hsatizab
 */
public class DataMatrix implements Cloneable, Serializable {

    protected DataMatrix(int nr, int nc) {
        nRows = nr;
        nColumns = nc;
        data = new float[nRows][nColumns];
    }
    
    protected DataMatrix() {
        nRows = 0;
        nColumns = 0;
        data = null;
    }
    
    public DataMatrix clone() throws CloneNotSupportedException {
        return (DataMatrix)super.clone();
    }
    
//    protected void initRows(int nr, int nc) {
//        nRows = nr;
//        nColumns = nc;
//        data = new float[nRows][];
//    }
    
    public void setRow(int ri, float[] d, boolean copy) {
        if (copy) {
            System.arraycopy(d, 0, data[ri], 0, nColumns);
        }
        else {
            data[ri] = d;
        }
    }

    public int getNRows() {
        return nRows;
    }
    
    public int getNColumns() {
        return nColumns;
    }
    
    public float[] getRow(int r) {
        return data[r];
    }
    
    /**
     * Transforms the maximum and minimum of each column.
     *
     * @param   nMin    the new minimum for the variable.
     * @param   nMax    the new maximum for the variable.
     */
    public void normalizeMaxMin(float nMin, float nMax) {
        float nRang = nMax - nMin;
        float nAvg = (nMax + nMin) / 2;
        
        for (int c = 0; c < nColumns; c++) {
            if (Float.compare(range[c], 0.0f) != 0) {
                for (int r = 0; r < nRows; r++) {
                    data[r][c] = nRang * ((data[r][c] - minimum[c]) / range[c]) + nMin;
                }
            }
            else {
                for (int r = 0; r < nRows; r++) {
                    data[r][c] = nAvg;
                }
            }
        }
    }

    /**
     * Transforms the average and range of each column.
     *
     * @param   nAvg    the new minimum for the variable.
     * @param   nRange    the new maximum for the variable.
     */
    public void normalizeAvgRange(float nAvg, float nRange) {
        float nMin;

        for (int c = 0; c < nColumns; c++) {
            nMin = minimum[c] + (nAvg - mean[c]);
            if (range[c] != 0) {
                for (int r = 0; r < nRows; r++) {
                    data[r][c] = nRange * ((data[r][c] - minimum[c]) / range[c]) + nMin;
                }
            }
            else {
                for (int r = 0; r < nRows; r++) {
                    data[r][c] = nAvg;
                }
            }
        }
    }

    /**
     * Transforms the average and range of each column.
     *
     * @param   nAvg    the new minimum for the variable.
     * @param   nRange    the new maximum for the variable.
     */
    public void normalizePlusMult(float toAdd, float toMult) {
        for (int c = 0; c < nColumns; c++) {
            for (int r = 0; r < nRows; r++) {
                data[r][c] = ((data[r][c] + toAdd) * toMult);
            }
        }
    }

    /**
     * Clips the maximum and minimum of each column.
     *
     * @param   nMin    the new minimum for the variable.
     * @param   nMax    the new maximum for the variable.
     */
    public void normalizeClipMinMax(float newMin, float newMax) {
        for (int c = 0; c < nColumns; c++) {
            for (int r = 0; r < nRows; r++) {
                if (data[r][c] < newMin) {
                    data[r][c] = newMin;
                }
                else {
                    if (data[r][c] > newMax) {
                        data[r][c] = newMax;
                    }
                }
            }
        }
    }


    /**
     * Computes the statistics of all the columns. Average, variance, minimum, maximum and range are calculated.
     * 
     * @see #getMean()
     * @see #getVar()
     * @see #getMin()
     * @see #getMax()
     * @see #getRange()
     */
    public void calcStats() {
        int c, r;

        mean = new float[nColumns];
        var = new float[nColumns];
        minimum = new float[nColumns];
        maximum = new float[nColumns];
        range = new float[nColumns];
        MSStot = new float[nColumns];

        if (nRows <= 1) {
            return;
        }
        
        float act;
        
        for (c = 0; c < nColumns; c++) {
            mean[c] = 0;
            var[c] = 0;
            minimum[c] = Float.MAX_VALUE;
            maximum[c] = Float.NEGATIVE_INFINITY;

            for (r = 0; r < nRows; r++){
                act = data[r][c];
                mean[c] += act;             //n* E(X)
                var[c] += (act*act);        //n * E(X^2)
                if (act < minimum[c]) {
                    minimum[c] = act;
                }
                if (act > maximum[c]) {
                    maximum[c] = act;
                }
            }
            mean[c] /= nRows;                  //E(X)
            var[c] /= nRows;                   //E(X^2)
            var[c] -= (mean[c]*mean[c]);    ////E(X^2) - E(X)^2
            range[c] = maximum[c] - minimum[c];

            for (r = 0; r < nRows; r++) {
                act = data[r][c];
                MSStot[c] += (act - mean[c]) * (act - mean[c]);
            }
            MSStot[c] /= this.nRows;
        }
    }

    /**
     * Returns the vector of average values.
     *
     * @return  the average of the columns.
     * @see #getVar()
     * @see #getMin()
     * @see #getMax()
     * @see #getRange()
     */
    public float[] getMean() {
        return mean;
    }
    
    /**
     * Returns the vector of variance.
     *
     * @return  the variance of the columns.
     * @see #getMean()
     * @see #getMin()
     * @see #getMax()
     * @see #getRange()
     */
    public float[] getVar() {
        return var;
    }

    /**
     * Returns the vector of the minimum values.
     *
     * @return  the minimum value of the columns.
     * @see #getMean()
     * @see #getVar()
     * @see #getMax()
     * @see #getRange()
     */
    public float[] getMin() {
        return minimum;
    }

    /**
     * Returns the vector of maximum values.
     *
     * @return  the maximum value of the columns.
     * @see #getMean()
     * @see #getVar()
     * @see #getMin()
     * @see #getRange()
     */
    public float[] getMax() {
        return maximum;
    }

    /**
     * Returns the vector of ranges.
     * 
     * @return the range of the columns.
     * @see #getMean()
     * @see #getVar()
     * @see #getMin()
     * @see #getMax()
     */
    public float[] getRange() {
        return range;
    }

    /**
     * Returns the vector of ranges.
     *
     * @return the SStot of the columns.
     * @see #getMean()
     * @see #getVar()
     * @see #getMin()
     * @see #getMax()
     * @see #getRange()
     */
    public float[] getMSStot() {
        return MSStot;
    }

    public String toString() {
        String rep = new String();
        
        for (int r = 0; r < nRows; r++) {
            for (int c = 0; c < nColumns; c++) {
                rep += String.valueOf(data[r][c]);
                if ((c+1) < nColumns) {
                    rep += "\t";
                }
            }
            rep += "\n";
        }
        
        return rep;
    }

    protected void copyStats(DataMatrix source) {
        mean = new float[nColumns];
        var = new float[nColumns];
        minimum = new float[nColumns];
        maximum = new float[nColumns];
        range = new float[nColumns];
        MSStot = new float[nColumns];
        
        System.arraycopy(source.mean, 0, mean, 0, nColumns);
        System.arraycopy(source.var, 0, var, 0, nColumns);
        System.arraycopy(source.minimum, 0, minimum, 0, nColumns);
        System.arraycopy(source.maximum, 0, maximum, 0, nColumns);
        System.arraycopy(source.range, 0, range, 0, nColumns);
        System.arraycopy(source.MSStot, 0, MSStot, 0, nColumns);
    }

    private int nRows;
    private int nColumns;
    private float [][] data;
    
    private float[] mean;
    private float[] var;
    private float[] minimum;
    private float[] maximum;
    private float[] range;
    private float[] MSStot;
}
